@@ -36,11 +36,20 @@ class Include(Module):
     # includes should happen before anything else
     priority = 0
 
+    # skip file flag
+    bypass_marker = "<!-- markup:markdown-end -->"
+
     def transform(self, data):
         transforms = []
 
         linenum = 0
         for line in data:
+            if line.strip() == self.bypass_marker:
+                for dropped in range(linenum, len(data)):
+                    transform = Transform(linenum=dropped, oper="drop")
+                    transforms.append(transform)
+                return transforms
+
             match = self.includere.search(line)
             if match:
                 includedata = self.include(match)
@@ -63,6 +72,9 @@ class Include(Module):
             linenum = 0
             includednum = 0
             for line in data:
+                if line.strip() == self.bypass_marker:
+                    return data[:linenum]
+
                 match = self.includere.search(line)
                 if match:
                     dirname = path.dirname(filename)
